# studio12.r solutions

# In going through these problems you should read all the
# comments and answer the questions as they are asked.

# **************************************************************#
# Problem 1. The trimmed mean.

# To compute the trimmed mean you first drop extreme values at
# either extreme of the data and then compute the mean of the
# remaining data points.

# Here is the data for this problem.
x = c(-40, -9, 0:24)

# PROBLEM 1(a). Use ?mean and read how the trim argument is used.

# PROBLEM 1(b). Use the mean function with the trim argument
# to compute the trimmed mean that trims 10% of the data from
# either end of the following data. Print out the result.
print(mean(x, trim=.1))

# PROBLEM 1(c). Compute the regular mean. Compare this with the
# trimmed mean with the mean.
print(mean(x))

# **************************************************************#
# Problem 2. Empirical bootstrap for location statistics.
# (From Problem 10.26 of Rice 3rd Edition.)

# In this problem you will be asked to write the code for
# a general purpose bootstrap function. First we give code
# to load and display the data. 

# Hampson and Walker made measurements of the heats of sublimation 
# for three metals:  platinum, iridium and rhodium. 

# Use this code to load their data
x.platinum=scan(file="platinum.txt")
x.iridium=scan(file="iridium.txt")
x.rhodium=scan(file="rhodium.txt")

# We define two functions for later use. Both of these
# compute a trimmed mean
trimmean1 = function(x) { return(mean(x,trim=.1))}
trimmean2 = function(x) { return(mean(x,trim=.2))}

# Here is a function to display the data in various ways
displayData = function(x) {
  # Plot a histogram, stem-and-leaf plot and a boxplot of the data
  hist(x)
  stem(x)
  boxplot(x)
  
  # Plot the observations in the order of the experiment.
  # This is a useful way to look for large anomolies.
  plot(x)  
  
  # Compute different statistics estimating the center/location
  # of the distribution:
  m = mean(x); 
  mt1 = trimmean1(x)
  mt2 = trimmean2(x)
  med = median(x); 
  cat('Mean =', m, 'trimmed mean (.1) =', mt1,
      'trimmed mean (.2) =', mt2, 'median =', med, '\n')
}


# PROBLEM 2(a). Fill in the code for the following function
# that is supposed to compute bootstrap confidence intervals
# for all our different locations statistics.
#
# The specifications are given within the function.

doEmpiricalBootstrapConfInt = function(x, statFunction,
                                      nboot, alpha, 
                                      statName) {
    # The arguments of the function have the following meanings.
    # x = data
    # statFunction is the statistic we want the 
    #   confidence interval around.
    # For us it's one of mean, trimmean1, trimmean2, median
    # nboot is the number of bootstrap samples to use
    # alpha is the significance level, i.e. make a 1-alpha 
    #   confidence interval.
    # statName is for printing purposes

    # This function should compute and print out the confidence
    # interval it computes. 

    nDataValues = length(x)
    thetahat = statFunction(x)

    # We'll do the bootstrap in a loop instead of using matrix
    # Initialize an array to hold the bootstap differences and 
    # populate it in a loop.
    deltastar = rep(0,nboot)  
    for (i in 1:nboot) {
        bssample = sample(x,nDataValues, replace=TRUE)
        deltastar[i] = statFunction(bssample) - thetahat
    }
  
    # Find the quantiles and confidence interval. Be sure 
    # to subtract in the correct order.
    criticalValues = quantile(deltastar, c(alpha/2, 1-alpha/2))
    CI = c(thetahat-criticalValues[2], thetahat-criticalValues[1])
    
    # Print the confidence interfal
    cat('Bootstrap confidence ', 1-alpha, ' interval for ', 
        statName, ' = [', CI[1], ', ', CI[2], ']\n')
}

# PROBLEM 2(b). Run the following code that makes use of
# the function you just wrote. Notice how the bootstrap 
# handles all the different statistics equally easily.

# We chose one dataset to look at and commented out the other 2.
x=x.platinum
#x=x.iridium
#x=x.rhodium

displayData(x)
doEmpiricalBootstrapConfInt(x, mean, nboot = 100, 
                            alpha = .1, statName = 'mean')
doEmpiricalBootstrapConfInt(x, trimmean1, nboot = 1000, 
                            alpha = .1,
                            statName = "trimmed mean (.1)")
doEmpiricalBootstrapConfInt(x, trimmean2, nboot = 1000, 
                            alpha = .1,
                            statName = "trimmed mean (.2)")
doEmpiricalBootstrapConfInt(x, median, nboot = 1000, 
                            alpha = .1, statName = "median")

# **************************************************************#
# PROBLEM 3. Financial data problem: linear regression.
# In this problem we will look at bivariate data and do linear regression.

# We'll use publically available data on the price of 
# several stocks over a 14 year period from 2000 to 2014. 
# We'll use a simplified version of the Capital Asset 
# Pricing Model or CAPM.

#-------------------------------------------#
# SAMPLE CODE. Here's sample code for computing and displaying
# the simple linear regression using the lm() function.
x = 1:10
y = x + rnorm(10,0,.5)
plot(x,y, col='orange', pch=19, cex=.6)
abline(h=0,v=0)

# Use lm() to do the fit. y ~ x is called a 'formula'
lmfit = lm(y ~ x)  # Note the order of y and x.

# Plot the regression line.
abline(lmfit,col='blue', lwd=2)

# Print a summary of the result of the regression
print(summary(lmfit))

#--------------------------------#
# Exploring the original data

# The original data is in the file studio12financialOriginal.csv. 
# We start with a quick exploration of the data.

#. The file contains daily prices or interest rates, over 
#  14 years, for  several stocks, bonds and a barrel of oil.
#. We will focus on:
#    SP500: Standard \& Poors stock index, the cost of a 
#    certain basket of stocks.
#    GE: General Electric.
#    DCOICWTICO: the price of a barrel of the benchmark West 
#    Texas crude oil.
#  We used studio12-prep.r to process the original file and 
#  produce studio12financialDaily.csv, which is the file we 
#  will use in this problem

# The following code loads and inpsects the financial data we need.
s12data.daily = read.csv('studio12financialDaily.csv')
print(head(s12data.daily))
print(tail(s12data.daily))
print(dim(s12data.daily))

# Extract columns
SP500.daily = s12data.daily[,"SP500.daily"]
GE.daily = s12data.daily[,"GE.daily"]
DCOILWTICO.daily = s12data.daily[,"DCOILWTICO.daily"]

#-------------#
# PROBLEM 3(a). In this problem we'll investigate how
# well the SP500 index predicts the price of GE

# (3a.i) Make a scatter plot of GE.daily vs SP500.daily.
# (3a.ii) Add axes lines through the origin.
# (3a.iii) Use the lm() function to do simple linear regression
#         on GE.daily vs SP500.daily.
# (3a.iv) Draw the regression line.
# (3a.v)  Print a summary of the result of the regression.
# (3a.vi) Look at the summary and using R help, google and pure
# thought try to understand what it says.

plot(SP500.daily,GE.daily, pch=19, cex=.3, col='orange')
abline(h=0,v=0)
lmfit = lm(GE.daily ~ SP500.daily)
abline(lmfit,col='blue', lwd=2)
title("Simple linear fit of GE.daily vs SP500,daily")
print(summary(lmfit))

# **************************************************************#
# PROBLEM 4. Anther boostrap  problem
# Find a 95% bootstrap confidence interval for the 
# difference of the means for the two columns of salaries.csv

# Here is code to load the two columns
salarydata = read.csv("salaries.csv")
head(salarydata)
salaries1 = salarydata[,"Salaries.1"]  # This is column 1
salaries2 = salarydata[,"Salaries.2"]  # This is column s

#4(a.i) Plot the data to see that it appears reasonable.
plot(salaries1, col="orange", type='l', lwd=2, ylab="Salaries")
lines(salaries2, col='blue', lwd=2)
title("Plots of salaries1 and salaries2")

#4(a.ii) Find the sample mean difference.
sampleMeanDiff = mean(salaries1) - mean(salaries2)

#4(a.iii) Generate a lot of bootstrap mean differences and use 
# them to compute delta*.
nboot = 5000
nsalaries1 = length(salaries1)
nsalaries2 = length(salaries2)

deltaStar = rep(0,nboot)
for (j in 1:nboot)
{
  salaries1Star = sample(salaries1, nsalaries1, replace=TRUE)
  salaries2Star = sample(salaries2, nsalaries2, replace=TRUE)  
  deltaStar[j] = (mean(salaries1Star) - mean(salaries2Star)) - sampleMeanDiff
}
# Pick out the 0.025 and 0.975 RIGHT CRITICAL VALUES of deltaStar
dstar_025 = quantile(deltaStar, 0.975)
dstar_975 = quantile(deltaStar, 0.025)

confint = sampleMeanDiff - c(dstar_025, dstar_975)

cat("Sample mean difference = ", sampleMeanDiff, "\n")
cat("95% bootstrap confidence interval = [", confint[1], ", ", confint[2], "]\n", sep="")

#----------------------------------------------#

doParametricBootstrapConfInt = function(x, statFunction,
                                       nboot, alpha, rdistribution, 
                                       statName) {
  # The arguments of the function have the following meanings.
  # x = data
  # statFunction is the statistic we want the 
  #   confidence interval around.
  # For us it's one of mean, trimmean1, trimmean2, median
  # nboot is the number of bootstrap samples to use
  # alpha is the significance level, i.e. make a 1-alpha 
  #   confidence interval.
  # statName is for printing purposes
  
  # rdistribution is a distribution R knows, like rbinom, rnorm,..
  
  # This function should compute and print out the confidence
  # interval it computes. 
  
  nDataValues = length(x)
  thetahat = statFunction(x)
  
  # We'll do the bootstrap in a loop instead of using matrix
  # Initialize an array to hold the bootstap differences and 
  # populate it in a loop.
  deltastar = rep(0,nboot)  
  for (i in 1:nboot) {
    # bssample = sample(x,nDataValues, replace=TRUE)
    bssample = rdistribution(1,nDataValues,thetahat)
    deltastar[i] = statFunction(bssample) - thetahat
  }
  
  # Find the quantiles and confidence interval. Be sure 
  # to subtract in the correct order.
  criticalValues = quantile(deltastar, c(alpha/2, 1-alpha/2))
  CI = c(thetahat-criticalValues[2], thetahat-criticalValues[1])
  
  # Print the confidence interfal
  cat('Bootstrap confidence ', 1-alpha, ' interval for ', 
      statName, ' = [', CI[1], ', ', CI[2], ']\n')
}