# studio12.r problems

# In going through these problems you should read all the
# comments and answer the questions as they are asked.

# **************************************************************#
# Problem 1. The trimmed mean.

# To compute the trimmed mean you first drop extreme values at
# either extreme of the data and then compute the mean of the
# remaining data points.

# Here is the data for this problem.
x = c(-40, -9, 0:24)

# PROBLEM 1(a). Use ?mean and read how the trim argument is used.

# PROBLEM 1(b). Use the mean function with the trim argument
# to compute the trimmed mean that trims 10% of the data from
# either end of the following data. Print out the result.

# PROBLEM 1(c). Compute the regular mean. Compare this with the
# trimmed mean with the mean.

# **************************************************************#
# Problem 2. Empirical bootstrap for location statistics.
# (From Problem 10.26 of Rice 3rd Edition.)

# In this problem you will be asked to write the code for
# a general purpose bootstrap function. First we give code
# to load and display the data. 

# Hampson and Walker made measurements of the heats of sublimation 
# for three metals:  platinum, iridium and rhodium. 

# Use this code to load their data
x.platinum=scan(file="platinum.txt")
x.iridium=scan(file="iridium.txt")
x.rhodium=scan(file="rhodium.txt")

# We define two functions for later use. Both of these
# compute a trimmed mean
trimmean1 = function(x) { return(mean(x,trim=.1))}
trimmean2 = function(x) { return(mean(x,trim=.2))}

# Here is a function to display the data in various ways
displayData = function(x) {
  # Plot a histogram, stem-and-leaf plot and a boxplot of the data
  hist(x)
  stem(x)
  boxplot(x)
  
  # Plot the observations in the order of the experiment.
  # This is a useful way to look for large anomolies.
  plot(x)  
  
  # Compute different statistics estimating the center/location
  # of the distribution:
  m = mean(x); 
  mt1 = trimmean1(x)
  mt2 = trimmean2(x)
  med = median(x); 
  cat('Mean =', m, 'trimmed mean (.1) =', mt1,
      'trimmed mean (.2) =', mt2, 'median =', med, '\n')
}


# PROBLEM 2(a). Fill in the code for the following function
# that is supposed to compute bootstrap confidence intervals
# for all our different locations statistics.
#
# The specifications are given within the function.

doEmpiricalBootstrapConfInt = function(x, statFunction,
                                      nboot, alpha, 
                                      statName) {
    # The arguments of the function have the following meanings.
    # x = data
    # statFunction is the statistic we want the 
    #   confidence interval around.
    # For us it's one of mean, trimmean1, trimmean2, median
    # nboot is the number of bootstrap samples to use
    # alpha is the significance level, i.e. make a 1-alpha 
    #   confidence interval.
    # statName is for printing purposes

    # This function should compute and print out the confidence
    # interval it computes. 

    nDataValues = length(x)
    thetahat = statFunction(x)
  
    # Add your code here.
  
}

# PROBLEM 2(b). Run the following code that makes use of
# the function you just wrote. Notice how the bootstrap 
# handles all the different statistics equally easily.

# We chose one dataset to look at and commented out the other 2.
x=x.platinum
#x=x.iridium
#x=x.rhodium

displayData(x)
doEmpiricalBootstrapConfInt(x, mean, nboot = 100, 
                            alpha = .1, statName = 'mean')
doEmpiricalBootstrapConfInt(x, trimmean1, nboot = 1000, 
                            alpha = .1,
                            statName = "trimmed mean (.1)")
doEmpiricalBootstrapConfInt(x, trimmean2, nboot = 1000, 
                            alpha = .1,
                            statName = "trimmed mean (.2)")
doEmpiricalBootstrapConfInt(x, median, nboot = 1000, 
                            alpha = .1, statName = "median")

# **************************************************************#
# PROBLEM 3. Financial data problem: linear regression.
# In this problem we will look at bivariate data and do linear regression.

# We'll use publically available data on the price of 
# several stocks over a 14 year period from 2000 to 2014. 
# We'll use a simplified version of the Capital Asset 
# Pricing Model or CAPM.

#-------------------------------------------#
# SAMPLE CODE. Here's sample code for computing and displaying
# the simple linear regression using the lm() function.
x = 1:10
y = x + rnorm(10,0,.5)
plot(x,y, col='orange', pch=19, cex=.6)
abline(h=0,v=0)

# Use lm() to do the fit. y ~ x is called a 'formula'
# Note the order of y and x.
lmfit = lm(y ~ x)

# Plot the regression line.
abline(lmfit,col='blue', lwd=2)

# Print a summary of the result of the regression
print(summary(lmfit))

#--------------------------------#
# Exploring the original data

# The original data is in the file studio12financialOriginal.csv. 
# We start with a quick exploration of the data.

#. The file contains daily prices or interest rates, over 
#  14 years, for  several stocks, bonds and a barrel of oil.
#. We will focus on:
#    SP500: Standard \& Poors stock index, the cost of a 
#    certain basket of stocks.
#    GE: General Electric.
#    DCOICWTICO: the price of a barrel of the benchmark West 
#    Texas crude oil.
#  We used studio12-prep.r to process the original file and 
#  produce studio12financialDaily.csv, which is the file we 
#  will use in this problem

# The following code loads and inpsects the financial data we need.
s12data.daily = read.csv('studio12financialDaily.csv')
print(head(s12data.daily))
print(tail(s12data.daily))
print(dim(s12data.daily))

# Extract columns
SP500.daily = s12data.daily[,"SP500.daily"]
GE.daily = s12data.daily[,"GE.daily"]
DCOILWTICO.daily = s12data.daily[,"DCOILWTICO.daily"]

#-------------#
# PROBLEM 3(a). In this problem we'll investigate how
# well the SP500 index predicts the price of GE

# (3a.i) Make a scatter plot of GE.daily vs SP500.daily.
# (3a.ii) Add axes lines through the origin.
# (3a.iii) Use the lm() function to do simple linear regression
#         on GE.daily vs SP500.daily.
# (3a.iv) Draw the regression line.
# (3a.v)  Print a summary of the result of the regression.
# (3a.vi) Look at the summary and using R help, google and pure
# thought try to understand what it says.

# **************************************************************#
# PROBLEM 4. Anther boostrap  problem
# Find a 95% bootstrap confidence interval for the 
# difference of the means for the two columns of salaries.csv

# Here is code to load the two columns
salarydata = read.csv("salaries.csv")
head(salarydata)
salaries1 = salarydata[,"Salaries.1"]  # This is column 1
salaries2 = salarydata[,"Salaries.2"]  # This is column s

#4(a.i) Plot the data to see that it appears reasonable.

#4(a.ii) Find the sample mean difference.

#4(a.iii) Generate a lot of bootstrap mean differences and use 
# them to compute delta*.
nboot = 5000
nsalaries1 = length(salaries1)
nsalaries2 = length(salaries2)


#----------------------------------------------#
